%twins
% Computer simulation of the Special Relativity (apparent) Twins Paradox.
% This accurately plots the displacement of a distant star from the
% a spacecraft in (i) the Earth frame and (ii) the spacecraft frame vs
% time, as measured in either frame.
%
% LAST UPDATED by Andy French Dec 2019

function twins

%Distance to star in light years (Earth frame)
d = 4;
%d = 10;

%Speed of spacecraft (as a fraction of the speed of light)
v = 4/5;
%v = 0.99;

%Waiting time at distant star before returning /years
twait = 1;

%Earth frame time interval
dt = 0.2;

%Fontsize for graphs
fsize = 14;

%

%Determine gamma factor
y = (1 - v^2 )^(-1/2);

%Determine time offset of earth frame time, as calculated from spacecraft
%frame, due to change of velocity from v to -v. Assume in slowing down to
%stop at the star, the Earth frame time gains half this offset.
%It will gain the second half accelerating up to v on the return trip,
%but that will be accounted for by the sign change of v to -v in the
%Lorentz transform.
offset = 2*y*(d*v/y);

%Determine t, x vectors in Earth frame, and hence tdash and xdash vectors
%in spacecraft frame using the Lorentz Transform. Assume clocks initially
%synchronized i.e. tdash(1) = t(1) = 0;

%Also compute the displacement of Earth, and Earth time, computed from
%spacecraft time and displacement of Earth, using the Lorenz Transform

%Going to star
t1 = 0 : dt : d/v;
x1 = d - v*t1;
tdash1 = t1/y;
xdash1 = x1/y;
tearth1 = y*( tdash1 - v*tdash1 );

%Waiting at star
t2 = d/v: dt : d/v + twait;
x2 = zeros( 1,length(t2) );
tdash2 = tdash1(end) + dt*( 0 : (length(t2) - 1) );
xdash2 = zeros( 1,length(tdash2) );
tearth2 = tearth1(end) + dt*( 0 : (length(t2) - 1) ) + offset/2;

%Returning from star
t3 = d/v+twait : dt : 2*d/v + twait;
x3 = v*( t3 - d/v - twait );
tdash3 = ( t3 - d/v - twait )/y + tdash2(end) ;
xdash3 = x3/y;
xdashearth3 = d/(v*y) - v*( tdash3 - tdash2(end) );
tearth3 = tearth2(end) + y*( tdash3 - tdash2(end) + v*xdashearth3 ) ;

%Assemble t,x vectors
t = [t1,t2,t3]; x = [x1,x2,x3]; 
tdash = [tdash1,tdash2,tdash3];
tearth = [tearth1,tearth2,tearth3];
xdash = [xdash1,xdash2,xdash3];

%

%Plot time vs space (t vs x) graphs

%t vs x
subplot( 2,2,1);
plot(x,t,'+');
xlabel('x /ly','fontsize',fsize); ylabel('t /yr','fontsize',fsize); grid on;
set(gca,'fontsize',fsize)
title(['d=',num2str(d),'ly, v/c=',num2str(v),', t_{wait}=',num2str(twait),'yr'],'fontsize',fsize);
xlimits = get(gca,'xlim'); ylimits = get(gca,'ylim'); 

%tdash vs xdash
subplot( 2,2,2);
plot(xdash,tdash,'+');
xlabel('xdash /ly','fontsize',fsize); ylabel('tdash /yr','fontsize',fsize); grid on;
set(gca,'fontsize',fsize); axis manual
set(gca,'xlim',xlimits,'ylim',ylimits);

%tdash vs t
subplot( 2,2,3);
plot(t,tdash,'+'); hold on; grid on;
plot(t,t,'r-');
legend({['\gamma=',num2str(y,3)],'Classical'},'fontsize',fsize,'location','northwest');
set(gca,'fontsize',fsize)
xlabel('t /yr','fontsize',fsize); ylabel('tdash /yr','fontsize',fsize);

%tearth and t vs tdash
subplot( 2,2,4);
plot(tdash,t,'r+',tdash,tearth); hold on; grid on;
legend({'t','tearth'},'location','northwest');
set(gca,'fontsize',fsize)
xlabel('tdash /yr','fontsize',fsize); ylabel('t or tearth /ly','fontsize',fsize);

%Print PNG and close
print( gcf,'twins.png','-dpng','-r300');
close(gcf);

%End of code